"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const child_process_1 = __importDefault(require("child_process"));
const stream_1 = require("stream");
const concat_stream_1 = __importDefault(require("concat-stream"));
const flamebearer_1 = __importDefault(require("./flamebearer"));
const tickprocessor = require.resolve('../tickprocessor');
// TODO: can remove this if not using Node
function jsonCleanUp() {
    return new stream_1.Transform({
        transform: (data, _, next) => {
            if (data.indexOf('Testing v8 version different from logging version') > -1) {
                next();
                return;
            }
            next(null, data);
        }
    });
}
/**
 * Takes in a v8 or node generated profile log and turn into flamegraph with flamebearer.
 * Generates files based on scenarioName: flamegraph (.html), JS data (.data.js), and error output (.err)
 *
 * @param {string} logFile Log file path and base name input.
 * @param {string} outFile File path and base name for file output. Extensions will be added for each type of file output.
 * @returns {Promise<OutputFiles>} Promise resolving to generated files.
 */
// TODO: is file output good enough here as an API? should this return programmatic results?
function generateFlamegraph(logFile, outFile) {
    // TODO: account for --windows, --unix and --mac arguments? what is the output difference with and without these args?
    // TODO: use cli
    const preprocessProc = child_process_1.default.spawn(process.execPath, [tickprocessor, '--preprocess', '-j', logFile]);
    const flamegraphFile = outFile + '.html';
    const dataFile = outFile + '.data.js';
    const errorFile = outFile + '.err';
    // TODO: return early?
    if (!fs_1.default.existsSync(path_1.default.dirname(outFile))) {
        console.error(`Directory does not exist: ${path_1.default.dirname(outFile)}`);
    }
    // TODO: this just silently fails if the output directory doesn't exist
    const flamebearerPromise = new Promise((resolve, reject) => {
        const outStream = concat_stream_1.default(preprocessed => {
            if (preprocessed.length > 0) {
                // This line writes intermediate streaming output for troubleshooting:
                // fs.writeFileSync(flamegraphfile.split('.html').join('.preprocessed.log'), preprocessed);
                const [flamegraph, data] = flamebearer_1.default(preprocessed);
                fs_1.default.writeFileSync(flamegraphFile, flamegraph);
                fs_1.default.writeFileSync(dataFile, data);
                resolve({ dataFile, flamegraphFile });
            }
        });
        const errStream = concat_stream_1.default(err => {
            if (err.length > 0) {
                fs_1.default.writeFileSync(errorFile, err);
                resolve({ errorFile });
            }
        });
        preprocessProc.stderr.pipe(jsonCleanUp()).pipe(errStream);
        preprocessProc.stdout.pipe(jsonCleanUp()).pipe(outStream);
    });
    return flamebearerPromise;
}
exports.generateFlamegraph = generateFlamegraph;
/**
 * Takes in a v8 or node generated profile log and generate a summary text file (.summary.log)
 *
 * @param {string} logfile Log file input.
 * @param {string} outfile File path and base name for file output. Extensions will be added for each type of file output.
 * @returns {string} Summary file path.
 */
function generateSummary(logfile, outfile) {
    // TODO: account for --windows, --unix and --mac arguments? what is the output difference with and without these args?
    // TODO: use cli
    const summaryProc = child_process_1.default.spawn(process.execPath, [tickprocessor, logfile]);
    const summaryFile = outfile + '.summary.log';
    const flamebearerPromise = new Promise((resolve, reject) => {
        const concatStream = concat_stream_1.default(preprocessed => {
            // TODO: this code just blindly assumes flamegraphfile has an html extension
            fs_1.default.writeFileSync(summaryFile, preprocessed);
            resolve(summaryFile);
        });
        summaryProc.stdout.pipe(jsonCleanUp()).pipe(concatStream);
    });
    return flamebearerPromise;
}
exports.generateSummary = generateSummary;
