"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const puppeteer_1 = __importDefault(require("puppeteer"));
const generate_1 = require("./flamegraph/generate");
const processData_1 = require("./analysis/processData");
;
;
;
;
;
function cook(scenarios, config) {
    return __awaiter(this, void 0, void 0, function* () {
        // const extraV8Flags = '--log-source-code --log-timer-events';
        // const extraV8Flags = '--log-source-code';
        const extraV8Flags = '';
        const outDir = config.outDir ? path_1.default.resolve(config.outDir) : process.cwd();
        const tempDir = config.tempDir ? path_1.default.resolve(config.tempDir) : process.cwd();
        const logFile = path_1.default.join(tempDir, '/puppeteer.log');
        console.log(`logFile: ${logFile}`);
        const browser = yield puppeteer_1.default.launch({
            headless: true,
            args: [
                '--flag-switches-begin',
                '--no-sandbox',
                '--js-flags=--logfile=' + logFile + ' --prof --jitless --no-opt ' + extraV8Flags,
                '--flag-switches-end'
            ]
        });
        // A log file is made for each new tab/page.
        // Make an initial page here to force generation of initial system logs that we are not concerned with.
        // This allows us to associate newly created log files with tests as we run each scenario in a new tab.
        yield browser.newPage();
        // TODO: Need to decide whether it's ok to run tests in parallel. Variance from results seems to indicate
        // not, but then again other things outside of our control will also affect CPU load and results.
        // Run tests sequentially for now, at least as a chance of getting more consistent results when run locally.
        const perfTests = {};
        for (const scenario of scenarios) {
            let logFile = yield runPerfTest(browser, scenario.scenario, scenario.name, tempDir);
            let logFileRef;
            if (scenario.reference) {
                logFileRef = yield runPerfTest(browser, scenario.reference, scenario.name, tempDir);
            }
            let outFileRef = path_1.default.join(outDir, `${scenario.name}_ref`);
            let outFile = path_1.default.join(outDir, `${scenario.name}`);
            perfTests[scenario.name] = {
                logFile,
                outFile,
                reference: {
                    logFile: logFileRef,
                    outFile: outFileRef
                }
            };
        }
        console.log('perfTests: ' + JSON.stringify(perfTests));
        // Clean up
        yield browser.close();
        const analyses = {};
        // Serialize a bunch of async generation of flamegraphs
        // TODO: need an API story here. how will users get output? data structures? files? both?
        for (const scenario of scenarios) {
            const result = perfTests[scenario.name];
            const analysis = {};
            analysis.files = yield generate_1.generateFlamegraph(result.logFile, result.outFile);
            // await generateSummary(result.logFile, result.outFile);
            if (result.reference.logFile) {
                analysis.reference = {};
                analysis.reference.files = yield generate_1.generateFlamegraph(result.reference.logFile, result.reference.outFile);
                analysis.files.regressionFile = path_1.default.join(outDir, `${scenario.name}.regression.txt`);
                // await generateSummary(result.logFileRef, result.outFileRef);    
            }
            processResults(analysis);
            analyses[scenario.name] = analysis;
        }
        console.log('analyses: ' + JSON.stringify(analyses));
        return analyses;
    });
}
exports.cook = cook;
;
/**
 *
 * @param {*} browser Launched puppeteer instance.
 * @param {string} testUrl Base URL supporting 'scenario' and 'iterations' query parameters.
 * @param {string} scenarioName Name of scenario that will be used with baseUrl.
 * @param {string} logDir Absolute path to output log profiles.
 * @returns {string} Log file path associated with test.
 */
function runPerfTest(browser, testUrl, scenarioName, logDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const logFilesBefore = fs_1.default.readdirSync(logDir);
        const page = yield browser.newPage();
        // Default timeout is 30 seconds. This is good for most tests except for problematic components like DocumentCardTitle.
        // Disable timeout for now and tweak to a maximum setting once server condtiions are better known.
        // TODO: argument? should probably default to 30 seconds
        page.setDefaultTimeout(0);
        const logFilesAfter = fs_1.default.readdirSync(logDir);
        const testLogFile = arr_diff(logFilesBefore, logFilesAfter);
        if (testLogFile.length !== 1) {
            // We have to be able to identify log file associated with tab. Throw error if we can't.
            // TODO: what should be the standard for erroring? console.error? throw? return failure?
            // TODO: make sure all async function calls have catch blocks
            // TODO: make sure invalid URLs (and other inputs) don't hang result
            throw new Error(`Could not determine log file for ${testUrl}. Log files detected: [ ${testLogFile} ]`);
        }
        console.log(`Starting test for ${scenarioName} at ${testUrl}`);
        console.time('Ran perf test in');
        yield page.goto(testUrl);
        console.timeEnd('Ran perf test in');
        console.log('testLogFile: ' + testLogFile[0]);
        yield page.close();
        return path_1.default.join(logDir, testLogFile[0]);
    });
}
/**
 * Create test summary based on test results.
 */
function processResults(analysis) {
    const dataFileAfter = analysis.files && analysis.files.dataFile;
    const dataFileBefore = analysis.reference && analysis.reference.files && analysis.reference.files.dataFile;
    if (dataFileAfter) {
        analysis.numTicks = getTicks(dataFileAfter);
    }
    if (dataFileBefore) {
        analysis.reference.numTicks = getTicks(dataFileBefore);
        if (dataFileAfter) {
            const regressionAnalysis = processData_1.checkForRegressions(dataFileBefore, dataFileAfter);
            analysis.isRegression = regressionAnalysis.isRegression;
            if (regressionAnalysis.isRegression) {
                console.log(`regressionFile: ${JSON.stringify(analysis)}`);
                fs_1.default.writeFileSync(analysis.files.regressionFile, regressionAnalysis.summary);
            }
        }
    }
    return analysis;
}
/**
 * Get ticks from flamegraph file.
 *
 * @param {string} resultsFile
 */
function getTicks(resultsFile) {
    const numTicks = fs_1.default
        .readFileSync(resultsFile, 'utf8')
        .toString()
        .match(/numTicks\s?\=\s?([0-9]+)/);
    if (numTicks && numTicks[1]) {
        return Number.parseInt(numTicks[1]);
    }
    else {
        console.log('Could not read numTicks from ' + resultsFile);
        return undefined;
    }
}
/**
 * Array diff utility that returns a list of elements that are not present in both arrays.
 *
 * @param {Array} a1 First array
 * @param {Array} a2 Second array
 */
function arr_diff(a1, a2) {
    let a = {};
    let diff = [];
    for (var i = 0; i < a1.length; i++) {
        a[a1[i]] = true;
    }
    for (var i = 0; i < a2.length; i++) {
        if (a[a2[i]]) {
            delete a[a2[i]];
        }
        else {
            a[a2[i]] = true;
        }
    }
    for (var k in a) {
        diff.push(k);
    }
    return diff;
}
exports.default = {
    cook
};
